from plaid.model.products import Products
from plaid.model.item_import_request import ItemImportRequest
from plaid.model.item_import_request_user_auth import ItemImportRequestUserAuth
from plaid.model.accounts_get_request import AccountsGetRequest
from plaid.model.deposit_switch_create_request import DepositSwitchCreateRequest
from plaid.model.deposit_switch_get_request import DepositSwitchGetRequest
from plaid.model.deposit_switch_token_create_request import DepositSwitchTokenCreateRequest

from tests.integration.util import (
    create_client,
)

access_token = None

def setup_module(module):
    client = create_client()
    # import an item
    at_request = ItemImportRequest(
        products=[Products('identity'), Products('auth')],
        user_auth=ItemImportRequestUserAuth(
            user_id='user_good',
            auth_token='pass_good'
        )
    )
    at_response = client.item_import(at_request)
    assert at_response['access_token'] is not None
    global access_token
    access_token = at_response['access_token']


def test_full_flow():
    client = create_client()
    # get account id
    account_id_request = AccountsGetRequest(
        access_token=access_token
    )
    account_id_response = client.accounts_get(account_id_request)
    assert len(account_id_response['accounts']) != 0
    target_account_id = account_id_response['accounts'][0]['account_id']

    # create deposit switch
    create_request = DepositSwitchCreateRequest(
        target_account_id=target_account_id,
        target_access_token=access_token
    )
    create_response = client.deposit_switch_create(create_request)
    assert create_response['deposit_switch_id'] is not None

    deposit_switch_id = create_response['deposit_switch_id']

    # get deposit switch
    get_request = DepositSwitchGetRequest(
        deposit_switch_id=deposit_switch_id
    )
    get_response = client.deposit_switch_get(get_request)
    assert get_response['deposit_switch_id'] is not None
    assert get_response['target_item_id'] is not None
    assert get_response['target_account_id'] is not None
    assert get_response['date_created'] is not None
    assert get_response['state'] is not None

    # create deposit switch token
    token_request = DepositSwitchTokenCreateRequest(
        deposit_switch_id=deposit_switch_id
    )
    token_response = client.deposit_switch_token_create(token_request)
    assert token_response['deposit_switch_token'] is not None
    assert token_response['deposit_switch_token_expiration_time'] is not None



"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)

def lazy_import():
    from plaid.model.location import Location
    from plaid.model.payment_meta import PaymentMeta
    globals()['Location'] = Location
    globals()['PaymentMeta'] = PaymentMeta


class TransactionBase(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('transaction_type',): {
            'DIGITAL': "digital",
            'PLACE': "place",
            'SPECIAL': "special",
            'UNRESOLVED': "unresolved",
        },
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'account_id': (str,),  # noqa: E501
            'amount': (float,),  # noqa: E501
            'iso_currency_code': (str, none_type,),  # noqa: E501
            'unofficial_currency_code': (str, none_type,),  # noqa: E501
            'date': (date,),  # noqa: E501
            'pending': (bool,),  # noqa: E501
            'transaction_id': (str,),  # noqa: E501
            'transaction_type': (str,),  # noqa: E501
            'pending_transaction_id': (str, none_type,),  # noqa: E501
            'category_id': (str, none_type,),  # noqa: E501
            'category': ([str], none_type,),  # noqa: E501
            'location': (Location,),  # noqa: E501
            'payment_meta': (PaymentMeta,),  # noqa: E501
            'account_owner': (str, none_type,),  # noqa: E501
            'name': (str,),  # noqa: E501
            'original_description': (str, none_type,),  # noqa: E501
            'merchant_name': (str, none_type,),  # noqa: E501
            'check_number': (str, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'account_id': 'account_id',  # noqa: E501
        'amount': 'amount',  # noqa: E501
        'iso_currency_code': 'iso_currency_code',  # noqa: E501
        'unofficial_currency_code': 'unofficial_currency_code',  # noqa: E501
        'date': 'date',  # noqa: E501
        'pending': 'pending',  # noqa: E501
        'transaction_id': 'transaction_id',  # noqa: E501
        'transaction_type': 'transaction_type',  # noqa: E501
        'pending_transaction_id': 'pending_transaction_id',  # noqa: E501
        'category_id': 'category_id',  # noqa: E501
        'category': 'category',  # noqa: E501
        'location': 'location',  # noqa: E501
        'payment_meta': 'payment_meta',  # noqa: E501
        'account_owner': 'account_owner',  # noqa: E501
        'name': 'name',  # noqa: E501
        'original_description': 'original_description',  # noqa: E501
        'merchant_name': 'merchant_name',  # noqa: E501
        'check_number': 'check_number',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, account_id, amount, iso_currency_code, unofficial_currency_code, date, pending, transaction_id, *args, **kwargs):  # noqa: E501
        """TransactionBase - a model defined in OpenAPI

        Args:
            account_id (str): The ID of the account in which this transaction occurred.
            amount (float): The settled value of the transaction, denominated in the account's currency, as stated in `iso_currency_code` or `unofficial_currency_code`. Positive values when money moves out of the account; negative values when money moves in. For example, debit card purchases are positive; credit card payments, direct deposits, and refunds are negative.
            iso_currency_code (str, none_type): The ISO-4217 currency code of the transaction. Always `null` if `unofficial_currency_code` is non-null.
            unofficial_currency_code (str, none_type): The unofficial currency code associated with the transaction. Always `null` if `iso_currency_code` is non-`null`. Unofficial currency codes are used for currencies that do not have official ISO currency codes, such as cryptocurrencies and the currencies of certain countries.  See the [currency code schema](https://plaid.com/docs/api/accounts#currency-code-schema) for a full listing of supported `iso_currency_code`s.
            date (date): For pending transactions, the date that the transaction occurred; for posted transactions, the date that the transaction posted. Both dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format ( `YYYY-MM-DD` ).
            pending (bool): When `true`, identifies the transaction as pending or unsettled. Pending transaction details (name, type, amount, category ID) may change before they are settled.
            transaction_id (str): The unique ID of the transaction. Like all Plaid identifiers, the `transaction_id` is case sensitive.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            transaction_type (str): Please use the `payment_channel` field, `transaction_type` will be deprecated in the future.  `digital:` transactions that took place online.  `place:` transactions that were made at a physical location.  `special:` transactions that relate to banks, e.g. fees or deposits.  `unresolved:` transactions that do not fit into the other three types. . [optional]  # noqa: E501
            pending_transaction_id (str, none_type): The ID of a posted transaction's associated pending transaction, where applicable.. [optional]  # noqa: E501
            category_id (str, none_type): The ID of the category to which this transaction belongs. See [Categories](https://plaid.com/docs/#category-overview).  If the `transactions` object was returned by an Assets endpoint such as `/asset_report/get/` or `/asset_report/pdf/get`, this field will only appear in an Asset Report with Insights.. [optional]  # noqa: E501
            category ([str], none_type): A hierarchical array of the categories to which this transaction belongs. See [Categories](https://plaid.com/docs/#category-overview).  If the `transactions` object was returned by an Assets endpoint such as `/asset_report/get/` or `/asset_report/pdf/get`, this field will only appear in an Asset Report with Insights.. [optional]  # noqa: E501
            location (Location): [optional]  # noqa: E501
            payment_meta (PaymentMeta): [optional]  # noqa: E501
            account_owner (str, none_type): The name of the account owner. This field is not typically populated and only relevant when dealing with sub-accounts.. [optional]  # noqa: E501
            name (str): The merchant name or transaction description.  If the `transactions` object was returned by a Transactions endpoint such as `/transactions/get`, this field will always appear. If the `transactions` object was returned by an Assets endpoint such as `/asset_report/get/` or `/asset_report/pdf/get`, this field will only appear in an Asset Report with Insights.. [optional]  # noqa: E501
            original_description (str, none_type): The string returned by the financial institution to describe the transaction. For transactions returned by `/transactions/get`, this field is in beta and will be omitted unless the client is both enrolled in the closed beta program and has set `options.include_original_description` to `true`.. [optional]  # noqa: E501
            merchant_name (str, none_type): The merchant name, as extracted by Plaid from the `name` field.. [optional]  # noqa: E501
            check_number (str, none_type): The check number of the transaction. This field is only populated for check transactions.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.account_id = account_id
        self.amount = amount
        self.iso_currency_code = iso_currency_code
        self.unofficial_currency_code = unofficial_currency_code
        self.date = date
        self.pending = pending
        self.transaction_id = transaction_id
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



import time
from plaid.model.country_code import CountryCode
from plaid.model.payment_initiation_payment_list_request import PaymentInitiationPaymentListRequest
from plaid.model.payment_initiation_payment_create_request import PaymentInitiationPaymentCreateRequest
from plaid.model.link_token_create_request_payment_initiation import LinkTokenCreateRequestPaymentInitiation
from plaid.model.link_token_create_request import LinkTokenCreateRequest
from plaid.model.link_token_create_request_user import LinkTokenCreateRequestUser
from plaid.model.payment_initiation_payment_get_request import PaymentInitiationPaymentGetRequest
from plaid.model.payment_initiation_payment_list_request import PaymentInitiationPaymentListRequest
from plaid.model.payment_initiation_recipient_list_request import PaymentInitiationRecipientListRequest
from plaid.model.payment_initiation_address import PaymentInitiationAddress
from plaid.model.payment_initiation_recipient_create_request import PaymentInitiationRecipientCreateRequest
from plaid.model.payment_initiation_recipient_get_request import PaymentInitiationRecipientGetRequest
from plaid.model.products import Products
from plaid.model.recipient_bacs_nullable import RecipientBACSNullable
from plaid.model.payment_amount import PaymentAmount

from tests.integration.util import create_client


def payments_after_recipient_creation(client, recipient_id):
    # list recipients
    request = PaymentInitiationRecipientListRequest()

    response = client.payment_initiation_recipient_list(request)
    assert response['recipients'] is not None
    assert len(response['recipients']) > 0

    # create payment
    request = PaymentInitiationPaymentCreateRequest(
        recipient_id=recipient_id,
        reference='TestPayment',
        amount=PaymentAmount(
            currency='GBP',
            value=100.00
        )
    )
    response = client.payment_initiation_payment_create(request)
    payment_id = response['payment_id']
    assert payment_id is not None
    assert response['status'] is not None

    # create link token
    request = LinkTokenCreateRequest(
        products=[Products('payment_initiation')],
        client_name='Plaid Test',
        country_codes=[CountryCode('GB')],
        language='en',
        user=LinkTokenCreateRequestUser(
            client_user_id=str(time.time())
        ),
        payment_initiation=LinkTokenCreateRequestPaymentInitiation(
            payment_id=payment_id
        )
    )
    response = client.link_token_create(request)
    assert response['link_token'] is not None
    assert response['expiration'] is not None

    # get payment
    request = PaymentInitiationPaymentGetRequest(
        payment_id=payment_id
    )
    response = client.payment_initiation_payment_get(request)
    assert response['payment_id'] is not None
    assert response['reference'] is not None
    assert response['amount'] is not None
    assert response['status'] is not None
    assert response['last_status_update'] is not None
    assert response['recipient_id'] is not None

    # list payments
    request = PaymentInitiationPaymentListRequest(
        count=10
    )
    response = client.payment_initiation_payment_list(request)
    assert response['payments'] is not None and len(response['payments']) > 0


def test_all_payment_routes_with_bacs():
    client = create_client()

    # create recipient
    request = PaymentInitiationRecipientCreateRequest(
        name='John Doe',
        bacs=RecipientBACSNullable(account='26207729', sort_code='560029'),
        address=PaymentInitiationAddress(
            street=['street name 999'],
            city='city',
            postal_code='99999',
            country='GB'
        )
    )
    response = client.payment_initiation_recipient_create(request)
    recipient_id = response['recipient_id']
    assert recipient_id is not None

    # get recipient
    request = PaymentInitiationRecipientGetRequest(
        recipient_id=recipient_id
    )
    response = client.payment_initiation_recipient_get(request)
    assert response['recipient_id'] is not None
    assert response['name'] is not None
    assert response['bacs'] is not None
    assert response['address'] is not None
    payments_after_recipient_creation(client, recipient_id)


def test_all_payment_routes_with_iban():
    client = create_client()

    # create recipient
    request = PaymentInitiationRecipientCreateRequest(
        name='John Doe',
        iban='GB33BUKB20201555555555',
        address=PaymentInitiationAddress(
            street=['street name 999'],
            city='city',
            postal_code='99999',
            country='GB'
        )
    )
    response = client.payment_initiation_recipient_create(request)
    recipient_id = response['recipient_id']
    assert recipient_id is not None

    # get recipient
    request = PaymentInitiationRecipientGetRequest(
        recipient_id=recipient_id
    )
    response = client.payment_initiation_recipient_get(request)
    assert response['recipient_id'] is not None
    assert response['name'] is not None
    assert response['iban'] is not None
    assert response['address'] is not None

    payments_after_recipient_creation(client, recipient_id)



"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)


class SignalEvaluateCoreAttributes(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    additional_properties_type = None

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            'unauthorized_transactions_count_7d': (int,),  # noqa: E501
            'unauthorized_transactions_count_30d': (int,),  # noqa: E501
            'unauthorized_transactions_count_60d': (int,),  # noqa: E501
            'unauthorized_transactions_count_90d': (int,),  # noqa: E501
            'nsf_overdraft_transactions_count_7d': (int,),  # noqa: E501
            'nsf_overdraft_transactions_count_30d': (int,),  # noqa: E501
            'nsf_overdraft_transactions_count_60d': (int,),  # noqa: E501
            'nsf_overdraft_transactions_count_90d': (int,),  # noqa: E501
            'days_since_first_plaid_connection': (int, none_type,),  # noqa: E501
            'plaid_connections_count_7d': (int, none_type,),  # noqa: E501
            'plaid_connections_count_30d': (int, none_type,),  # noqa: E501
            'total_plaid_connections_count': (int, none_type,),  # noqa: E501
            'is_savings_or_money_market_account': (bool,),  # noqa: E501
            'total_credit_transactions_amount_10d': (float,),  # noqa: E501
            'total_debit_transactions_amount_10d': (float,),  # noqa: E501
            'p50_credit_transactions_amount_28d': (float, none_type,),  # noqa: E501
            'p50_debit_transactions_amount_28d': (float, none_type,),  # noqa: E501
            'p95_credit_transactions_amount_28d': (float, none_type,),  # noqa: E501
            'p95_debit_transactions_amount_28d': (float, none_type,),  # noqa: E501
            'days_with_negative_balance_count_90d': (int, none_type,),  # noqa: E501
            'p90_eod_balance_30d': (float, none_type,),  # noqa: E501
            'p90_eod_balance_60d': (float, none_type,),  # noqa: E501
            'p90_eod_balance_90d': (float, none_type,),  # noqa: E501
            'p10_eod_balance_30d': (float, none_type,),  # noqa: E501
            'p10_eod_balance_60d': (float, none_type,),  # noqa: E501
            'p10_eod_balance_90d': (float, none_type,),  # noqa: E501
            'available_balance': (float, none_type,),  # noqa: E501
            'current_balance': (float, none_type,),  # noqa: E501
            'balance_last_updated': (datetime, none_type,),  # noqa: E501
            'phone_change_count_28d': (int, none_type,),  # noqa: E501
            'phone_change_count_90d': (int, none_type,),  # noqa: E501
            'email_change_count_28d': (int, none_type,),  # noqa: E501
            'email_change_count_90d': (int, none_type,),  # noqa: E501
            'address_change_count_28d': (int, none_type,),  # noqa: E501
            'address_change_count_90d': (int, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'unauthorized_transactions_count_7d': 'unauthorized_transactions_count_7d',  # noqa: E501
        'unauthorized_transactions_count_30d': 'unauthorized_transactions_count_30d',  # noqa: E501
        'unauthorized_transactions_count_60d': 'unauthorized_transactions_count_60d',  # noqa: E501
        'unauthorized_transactions_count_90d': 'unauthorized_transactions_count_90d',  # noqa: E501
        'nsf_overdraft_transactions_count_7d': 'nsf_overdraft_transactions_count_7d',  # noqa: E501
        'nsf_overdraft_transactions_count_30d': 'nsf_overdraft_transactions_count_30d',  # noqa: E501
        'nsf_overdraft_transactions_count_60d': 'nsf_overdraft_transactions_count_60d',  # noqa: E501
        'nsf_overdraft_transactions_count_90d': 'nsf_overdraft_transactions_count_90d',  # noqa: E501
        'days_since_first_plaid_connection': 'days_since_first_plaid_connection',  # noqa: E501
        'plaid_connections_count_7d': 'plaid_connections_count_7d',  # noqa: E501
        'plaid_connections_count_30d': 'plaid_connections_count_30d',  # noqa: E501
        'total_plaid_connections_count': 'total_plaid_connections_count',  # noqa: E501
        'is_savings_or_money_market_account': 'is_savings_or_money_market_account',  # noqa: E501
        'total_credit_transactions_amount_10d': 'total_credit_transactions_amount_10d',  # noqa: E501
        'total_debit_transactions_amount_10d': 'total_debit_transactions_amount_10d',  # noqa: E501
        'p50_credit_transactions_amount_28d': 'p50_credit_transactions_amount_28d',  # noqa: E501
        'p50_debit_transactions_amount_28d': 'p50_debit_transactions_amount_28d',  # noqa: E501
        'p95_credit_transactions_amount_28d': 'p95_credit_transactions_amount_28d',  # noqa: E501
        'p95_debit_transactions_amount_28d': 'p95_debit_transactions_amount_28d',  # noqa: E501
        'days_with_negative_balance_count_90d': 'days_with_negative_balance_count_90d',  # noqa: E501
        'p90_eod_balance_30d': 'p90_eod_balance_30d',  # noqa: E501
        'p90_eod_balance_60d': 'p90_eod_balance_60d',  # noqa: E501
        'p90_eod_balance_90d': 'p90_eod_balance_90d',  # noqa: E501
        'p10_eod_balance_30d': 'p10_eod_balance_30d',  # noqa: E501
        'p10_eod_balance_60d': 'p10_eod_balance_60d',  # noqa: E501
        'p10_eod_balance_90d': 'p10_eod_balance_90d',  # noqa: E501
        'available_balance': 'available_balance',  # noqa: E501
        'current_balance': 'current_balance',  # noqa: E501
        'balance_last_updated': 'balance_last_updated',  # noqa: E501
        'phone_change_count_28d': 'phone_change_count_28d',  # noqa: E501
        'phone_change_count_90d': 'phone_change_count_90d',  # noqa: E501
        'email_change_count_28d': 'email_change_count_28d',  # noqa: E501
        'email_change_count_90d': 'email_change_count_90d',  # noqa: E501
        'address_change_count_28d': 'address_change_count_28d',  # noqa: E501
        'address_change_count_90d': 'address_change_count_90d',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """SignalEvaluateCoreAttributes - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            unauthorized_transactions_count_7d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to unauthorized transactions over the past 7 days from the account that will be debited.. [optional]  # noqa: E501
            unauthorized_transactions_count_30d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to unauthorized transactions over the past 30 days from the account that will be debited.. [optional]  # noqa: E501
            unauthorized_transactions_count_60d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to unauthorized transactions over the past 60 days from the account that will be debited.. [optional]  # noqa: E501
            unauthorized_transactions_count_90d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to unauthorized transactions over the past 90 days from the account that will be debited.. [optional]  # noqa: E501
            nsf_overdraft_transactions_count_7d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to non-sufficient funds/overdrafts over the past 7 days from the account that will be debited.. [optional]  # noqa: E501
            nsf_overdraft_transactions_count_30d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to non-sufficient funds/overdrafts over the past 30 days from the account that will be debited.. [optional]  # noqa: E501
            nsf_overdraft_transactions_count_60d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to non-sufficient funds/overdrafts over the past 60 days from the account that will be debited.. [optional]  # noqa: E501
            nsf_overdraft_transactions_count_90d (int): We parse and analyze historical transaction metadata to identify the number of possible past returns due to non-sufficient funds/overdrafts over the past 90 days from the account that will be debited.. [optional]  # noqa: E501
            days_since_first_plaid_connection (int, none_type): The number of days since the first time the Item was connected to an application via Plaid. [optional]  # noqa: E501
            plaid_connections_count_7d (int, none_type): The number of times the Item has been connected to applications via Plaid over the past 7 days. [optional]  # noqa: E501
            plaid_connections_count_30d (int, none_type): The number of times the Item has been connected to applications via Plaid over the past 30 days. [optional]  # noqa: E501
            total_plaid_connections_count (int, none_type): The total number of times the Item has been connected to applications via Plaid. [optional]  # noqa: E501
            is_savings_or_money_market_account (bool): Indicates if the ACH transaction funding account is a savings/money market account. [optional]  # noqa: E501
            total_credit_transactions_amount_10d (float): The total credit (inflow) transaction amount over the past 10 days from the account that will be debited. [optional]  # noqa: E501
            total_debit_transactions_amount_10d (float): The total debit (outflow) transaction amount over the past 10 days from the account that will be debited. [optional]  # noqa: E501
            p50_credit_transactions_amount_28d (float, none_type): The 50th percentile of all credit (inflow) transaction amounts over the past 28 days from the account that will be debited. [optional]  # noqa: E501
            p50_debit_transactions_amount_28d (float, none_type): The 50th percentile of all debit (outflow) transaction amounts over the past 28 days from the account that will be debited. [optional]  # noqa: E501
            p95_credit_transactions_amount_28d (float, none_type): The 95th percentile of all credit (inflow) transaction amounts over the past 28 days from the account that will be debited. [optional]  # noqa: E501
            p95_debit_transactions_amount_28d (float, none_type): The 95th percentile of all debit (outflow) transaction amounts over the past 28 days from the account that will be debited. [optional]  # noqa: E501
            days_with_negative_balance_count_90d (int, none_type): The number of days within the past 90 days when the account that will be debited had a negative end-of-day available balance. [optional]  # noqa: E501
            p90_eod_balance_30d (float, none_type): The 90th percentile of the end-of-day available balance over the past 30 days of the account that will be debited. [optional]  # noqa: E501
            p90_eod_balance_60d (float, none_type): The 90th percentile of the end-of-day available balance over the past 60 days of the account that will be debited. [optional]  # noqa: E501
            p90_eod_balance_90d (float, none_type): The 90th percentile of the end-of-day available balance over the past 90 days of the account that will be debited. [optional]  # noqa: E501
            p10_eod_balance_30d (float, none_type): The 10th percentile of the end-of-day available balance over the past 30 days of the account that will be debited. [optional]  # noqa: E501
            p10_eod_balance_60d (float, none_type): The 10th percentile of the end-of-day available balance over the past 60 days of the account that will be debited. [optional]  # noqa: E501
            p10_eod_balance_90d (float, none_type): The 10th percentile of the end-of-day available balance over the past 90 days of the account that will be debited. [optional]  # noqa: E501
            available_balance (float, none_type): Available balance, as of the `balance_last_updated` time. The available balance is the current balance less any outstanding holds or debits that have not yet posted to the account.. [optional]  # noqa: E501
            current_balance (float, none_type): Current balance, as of the `balance_last_updated` time. The current balance is the total amount of funds in the account.. [optional]  # noqa: E501
            balance_last_updated (datetime, none_type): Timestamp in [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DDTHH:mm:ssZ) indicating the last time that the balance for the given account has been updated.. [optional]  # noqa: E501
            phone_change_count_28d (int, none_type): The number of times the account's phone numbers on file have changed over the past 28 days. [optional]  # noqa: E501
            phone_change_count_90d (int, none_type): The number of times the account's phone numbers on file have changed over the past 90 days. [optional]  # noqa: E501
            email_change_count_28d (int, none_type): The number of times the account's email addresses on file have changed over the past 28 days. [optional]  # noqa: E501
            email_change_count_90d (int, none_type): The number of times the account's email addresses on file have changed over the past 90 days. [optional]  # noqa: E501
            address_change_count_28d (int, none_type): The number of times the account's addresses on file have changed over the past 28 days. [optional]  # noqa: E501
            address_change_count_90d (int, none_type): The number of times the account's addresses on file have changed over the past 90 days. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



import time
from plaid.model.account_subtype import AccountSubtype
from plaid.model.account_subtypes import AccountSubtypes
from plaid.model.country_code import CountryCode
from plaid.model.depository_filter import DepositoryFilter
from plaid.model.link_token_account_filters import LinkTokenAccountFilters
from plaid.model.link_token_create_request import LinkTokenCreateRequest
from plaid.model.link_token_create_request_user import LinkTokenCreateRequestUser
from plaid.model.link_token_get_request import LinkTokenGetRequest
from plaid.model.products import Products
from tests.integration.util import create_client

CLIENT_NAME = 'Plaid Test'


def test_link_token_create_required():
    client = create_client()

    request = LinkTokenCreateRequest(
        products=[Products('auth'), Products('transactions')],
        client_name=CLIENT_NAME,
        country_codes=[CountryCode('GB')],
        language='en',
        user=LinkTokenCreateRequestUser(
            client_user_id=str(time.time())
        )
    )
    # create link token
    response = client.link_token_create(request)

    # assert on response
    assert response['link_token'] is not None
    assert response['expiration'] is not None


def test_link_token_create_optional():
    client = create_client()

    # build the configs
    request = LinkTokenCreateRequest(
        products=[Products('auth'), Products('transactions')],
        client_name=CLIENT_NAME,
        country_codes=[CountryCode('GB')],
        language='en',
        webhook='https://sample-webhook-uri.com',
        link_customization_name='default',
        account_filters=LinkTokenAccountFilters(
            depository=DepositoryFilter(
                account_subtypes=AccountSubtypes(
                    [AccountSubtype('checking'), AccountSubtype('savings')])
            )
        ),
        user=LinkTokenCreateRequestUser(
            client_user_id=str(time.time())
        )
    )

    # create link token
    response = client.link_token_create(request)

    # assert on response
    assert response['link_token'] is not None
    assert response['expiration'] is not None


def test_link_token_create_and_get():
    client = create_client()

    # build the configs
    request = LinkTokenCreateRequest(
        products=[Products('auth'), Products('transactions')],
        client_name=CLIENT_NAME,
        country_codes=[CountryCode('GB')],
        language='en',
        webhook='https://sample-webhook-uri.com',
        link_customization_name='default',
        account_filters=LinkTokenAccountFilters(
            depository=DepositoryFilter(
                account_subtypes=AccountSubtypes(
                    [AccountSubtype('checking'), AccountSubtype('savings')])
            )
        ),
        user=LinkTokenCreateRequestUser(
            client_user_id=str(time.time())
        )
    )
    # create link token
    createResponse = client.link_token_create(request)

    # assert on response
    assert createResponse['link_token'] is not None
    assert createResponse['expiration'] is not None

    getRequest = LinkTokenGetRequest(
        link_token=createResponse['link_token']
    )
    getResponse = client.link_token_get(getRequest)
    assert getResponse['link_token'] == createResponse['link_token']
    assert getResponse['created_at'] is not None
    assert getResponse['expiration'] is not None

    assert getResponse['metadata'] is not None
    assert (getResponse['metadata']['initial_products'] ==
            [Products('auth'), Products('transactions')])
    assert (getResponse['metadata']['webhook'] ==
            'https://sample-webhook-uri.com')
    assert getResponse['metadata']['country_codes'] == [
        CountryCode('GB')]
    assert getResponse['metadata']['language'] == 'en'
    assert getResponse['metadata']['account_filters']['depository']['account_subtypes'] == \
        AccountSubtypes([AccountSubtype('checking'), AccountSubtype('savings')])
    assert getResponse['metadata']['client_name'] == CLIENT_NAME



import time
import json
import os
import plaid
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.asset_report_create_request import AssetReportCreateRequest
from plaid.model.asset_report_create_request_options import AssetReportCreateRequestOptions
from plaid.model.asset_report_filter_request import AssetReportFilterRequest
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.item_remove_request import ItemRemoveRequest
from plaid.model.asset_report_user import AssetReportUser
from plaid.model.asset_report_pdf_get_request import AssetReportPDFGetRequest
from plaid.model.asset_report_get_request import AssetReportGetRequest
from plaid.model.asset_report_refresh_request import AssetReportRefreshRequest
from plaid.model.asset_report_audit_copy_create_request import AssetReportAuditCopyCreateRequest
from plaid.model.asset_report_audit_copy_get_request import AssetReportAuditCopyGetRequest
from plaid.model.asset_report_audit_copy_remove_request import AssetReportAuditCopyRemoveRequest
from plaid.model.asset_report_remove_request import AssetReportRemoveRequest

from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
)

client_id = os.environ['CLIENT_ID']

access_token = None

def setup_module(module):
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('assets')],
    )
    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )
    exchange_response = client.item_public_token_exchange(exchange_request)
    global access_token
    access_token = exchange_response['access_token']


def teardown_module(module):
    client = create_client()
    ir_request = ItemRemoveRequest(
        access_token=access_token
    )
    client.item_remove(ir_request)


def test_full_flow():
    client = create_client()

    # create an asset report for one item
    request = AssetReportCreateRequest(
        access_tokens=[access_token],
        days_requested=60,
        options=AssetReportCreateRequestOptions(
            webhook='https://www.example.com',
            client_report_id='123',
            user=AssetReportUser(
                client_user_id='789',
                first_name='Jane',
                middle_name='Leah',
                last_name='Doe',
                ssn='123-45-6789',
                phone_number='(555) 123-4567',
                email='jane.doe@example.com',
            )
        )
    )

    response = client.asset_report_create(request)
    asset_report_token = response['asset_report_token']
    asset_report_id = response['asset_report_id']
    assert asset_report_token is not None
    assert asset_report_id is not None

    # retrieve the asset report
    response = poll_for_asset_report(client, asset_report_token)
    report = response['report']
    assert report is not None

    # retrieve the asset report as an Asset Report with Insights
    request = AssetReportGetRequest(
        asset_report_token=asset_report_token,
        include_insights=True
    )
    response = client.asset_report_get(request)
    report = response['report']
    assert report is not None

    # The transactions in an Asset Report with Insights should have a non-null
    # `name` (when available).
    assert (
        name_exists_for_some_transaction(report))

    # retrieve the asset report as a PDF
    request = AssetReportPDFGetRequest(
        asset_report_token=asset_report_token,
    )
    pdf = client.asset_report_pdf_get(request)
    assert pdf is not None

    # create a filtered copy of the asset report
    account_ids_to_exclude = [report['items'][0]['accounts'][0]['account_id']]
    request = AssetReportFilterRequest(
        asset_report_token=asset_report_token,
        account_ids_to_exclude=account_ids_to_exclude
    )
    response = client.asset_report_filter(request)
    assert response['asset_report_token'] is not None

    # create a refreshed copy of the asset report
    request = AssetReportRefreshRequest(
        asset_report_token=asset_report_token,
        days_requested=10
    )
    response = client.asset_report_refresh(request)
    assert response['asset_report_token'] is not None

    # create an audit copy
    request = AssetReportAuditCopyCreateRequest(
        asset_report_token=asset_report_token,
        auditor_id=client_id
    )

    response = client.asset_report_audit_copy_create(request)
    audit_copy_token = response['audit_copy_token']
    assert audit_copy_token is not None

    # get the audit copy
    request = AssetReportAuditCopyGetRequest(
        audit_copy_token=audit_copy_token,
    )
    response = client.asset_report_audit_copy_get(request)
    audit_copy = response['report']
    assert audit_copy is not None

    # remove the audit copy token
    request = AssetReportAuditCopyRemoveRequest(
        audit_copy_token=audit_copy_token,
    )
    response = client.asset_report_audit_copy_remove(request)
    removed = response['removed']
    assert removed

    # remove the asset report
    request = AssetReportRemoveRequest(
        asset_report_token=asset_report_token
    )
    response = client.asset_report_remove(request)
    removed = response['removed']
    assert removed


def name_exists_for_some_transaction(report):
    for account in report['items'][0]['accounts']:
        if len(account['transactions']) > 0:
            return (account['transactions'][0]['name'] is not None)
    return False


def poll_for_asset_report(client, asset_report_token, retries=20):
    try:
        request = AssetReportGetRequest(
            asset_report_token=asset_report_token,
        )
        return client.asset_report_get(request)
    except plaid.ApiException as e:
        response = json.loads(e.body)
        if response['error_code'] == 'PRODUCT_NOT_READY' and retries > 0:
            time.sleep(1)
            return poll_for_asset_report(
                client,
                asset_report_token,
                retries - 1)

        raise e



'''
Webhook tests
'''

from tests.integration.util import (
    create_client,
    WEBHOOK_VERIFICATION_KEY_ID
)

from plaid.model.webhook_verification_key_get_request import WebhookVerificationKeyGetRequest

def test_get_webhook_verification_key():
    client = create_client()

    request = WebhookVerificationKeyGetRequest(
        key_id=WEBHOOK_VERIFICATION_KEY_ID
    )
    response = client.webhook_verification_key_get(request)
    assert response['key'] is not None
    assert response['key']['alg'] is not None
    assert response['key']['crv'] is not None
    assert response['key']['kid'] is not None
    assert response['key']['kty'] is not None
    assert response['key']['use'] is not None
    assert response['key']['x'] is not None
    assert response['key']['y'] is not None


from plaid.model.country_code import CountryCode
from plaid.model.products import Products
from plaid.model.institutions_get_request import InstitutionsGetRequest
from plaid.model.institutions_get_request_options import InstitutionsGetRequestOptions
from plaid.model.institutions_get_by_id_request import InstitutionsGetByIdRequest
from plaid.model.institutions_get_by_id_request_options import InstitutionsGetByIdRequestOptions
from plaid.model.institutions_search_request import InstitutionsSearchRequest
from plaid.model.institutions_search_request_options import InstitutionsSearchRequestOptions

from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
    SANDBOX_INSTITUTION_NAME,
)

def test_get():
    client = create_client()
    request = InstitutionsGetRequest(
        country_codes=[CountryCode('US')],
        count=3,
        offset=1
    )
    response = client.institutions_get(request)
    assert len(response['institutions']) == 3


def test_get_with_include_optional_metadata():
    client = create_client()
    request = InstitutionsGetRequest(
        country_codes=[CountryCode('US')],
        count=3,
        offset=1,
        options=InstitutionsGetRequestOptions(
            include_optional_metadata=True
        )
    )
    response = client.institutions_get(request)

    assert len(response['institutions']) == 3
    assert len(response['institutions'][0]['url']) > 0
    assert len(response['institutions'][0]['primary_color']) > 0

    assert len(response['institutions'][1]['url']) > 0
    assert len(response['institutions'][1]['primary_color']) > 0

    assert len(response['institutions'][2]['url']) > 0
    assert len(response['institutions'][2]['primary_color']) > 0


def test_get_by_id():
    client = create_client()
    request = InstitutionsGetByIdRequest(
        institution_id=SANDBOX_INSTITUTION,
        country_codes=[CountryCode('US')]
    )
    response = client.institutions_get_by_id(request)
    assert response['institution']['institution_id'] == SANDBOX_INSTITUTION


def test_get_by_id_with_include_optional_metadata():
    client = create_client()
    request = InstitutionsGetByIdRequest(
        institution_id=SANDBOX_INSTITUTION,
        country_codes=[CountryCode('US')],
        options=InstitutionsGetByIdRequestOptions(
            include_optional_metadata=True
        )
    )

    response = client.institutions_get_by_id(request)
    assert response['institution']['institution_id'] == SANDBOX_INSTITUTION
    assert response['institution']['url'] is not None

def test_search_with_products():
    client = create_client()
    request = InstitutionsSearchRequest(
        query=SANDBOX_INSTITUTION_NAME,
        products=[Products('transactions')],
        country_codes=[CountryCode('US')],
    )
    response = client.institutions_search(request)
    assert len(response['institutions']) >= 1


def test_search_with_include_optional_metadata():
    client = create_client()
    request = InstitutionsSearchRequest(
        query=SANDBOX_INSTITUTION_NAME,
        products=[Products('transactions')],
        country_codes=[CountryCode('US')],
        options=InstitutionsSearchRequestOptions(
            include_optional_metadata=True
        )
    )
    response = client.institutions_search(request)
    assert len(response['institutions']) >= 1
    assert response['institutions'][0]['url'] is not None


"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)


class AccountBalance(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            'available': (float, none_type,),  # noqa: E501
            'current': (float, none_type,),  # noqa: E501
            'limit': (float, none_type,),  # noqa: E501
            'iso_currency_code': (str, none_type,),  # noqa: E501
            'unofficial_currency_code': (str, none_type,),  # noqa: E501
            'last_updated_datetime': (datetime, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'available': 'available',  # noqa: E501
        'current': 'current',  # noqa: E501
        'limit': 'limit',  # noqa: E501
        'iso_currency_code': 'iso_currency_code',  # noqa: E501
        'unofficial_currency_code': 'unofficial_currency_code',  # noqa: E501
        'last_updated_datetime': 'last_updated_datetime',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, available, current, limit, iso_currency_code, unofficial_currency_code, *args, **kwargs):  # noqa: E501
        """AccountBalance - a model defined in OpenAPI

        Args:
            available (float, none_type): The amount of funds available to be withdrawn from the account, as determined by the financial institution.  For `credit`-type accounts, the `available` balance typically equals the `limit` less the `current` balance, less any pending outflows plus any pending inflows.  For `depository`-type accounts, the `available` balance typically equals the `current` balance less any pending outflows plus any pending inflows. For `depository`-type accounts, the `available` balance does not include the overdraft limit.  For `investment`-type accounts (or `brokerage`-type accounts for API versions 2018-05-22 and earlier), the `available` balance is the total cash available to withdraw as presented by the institution.  Note that not all institutions calculate the `available`  balance. In the event that `available` balance is unavailable, Plaid will return an `available` balance value of `null`.  Available balance may be cached and is not guaranteed to be up-to-date in realtime unless the value was returned by `/accounts/balance/get`.  If `current` is `null` this field is guaranteed not to be `null`.
            current (float, none_type): The total amount of funds in or owed by the account.  For `credit`-type accounts, a positive balance indicates the amount owed; a negative amount indicates the lender owing the account holder.  For `loan`-type accounts, the current balance is the principal remaining on the loan, except in the case of student loan accounts at Sallie Mae (`ins_116944`). For Sallie Mae student loans, the account's balance includes both principal and any outstanding interest.  For `investment`-type accounts (or `brokerage`-type accounts for API versions 2018-05-22 and earlier), the current balance is the total value of assets as presented by the institution.  Note that balance information may be cached unless the value was returned by `/accounts/balance/get`; if the Item is enabled for Transactions, the balance will be at least as recent as the most recent Transaction update. If you require realtime balance information, use the `available` balance as provided by `/accounts/balance/get`.  When returned by `/accounts/balance/get`, this field may be `null`. When this happens, `available` is guaranteed not to be `null`.
            limit (float, none_type): For `credit`-type accounts, this represents the credit limit.  For `depository`-type accounts, this represents the pre-arranged overdraft limit, which is common for current (checking) accounts in Europe.  In North America, this field is typically only available for `credit`-type accounts.
            iso_currency_code (str, none_type): The ISO-4217 currency code of the balance. Always null if `unofficial_currency_code` is non-null.
            unofficial_currency_code (str, none_type): The unofficial currency code associated with the balance. Always null if `iso_currency_code` is non-null. Unofficial currency codes are used for currencies that do not have official ISO currency codes, such as cryptocurrencies and the currencies of certain countries.  See the [currency code schema](https://plaid.com/docs/api/accounts#currency-code-schema) for a full listing of supported `unofficial_currency_code`s.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            last_updated_datetime (datetime, none_type): Timestamp in [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (`YYYY-MM-DDTHH:mm:ssZ`) indicating the last time that the balance for the given account has been updated  This is currently only provided when the `min_last_updated_datetime` is passed when calling `/accounts/balance/get` for `ins_128026` (Capital One).. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.available = available
        self.current = current
        self.limit = limit
        self.iso_currency_code = iso_currency_code
        self.unofficial_currency_code = unofficial_currency_code
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)

def lazy_import():
    from plaid.model.country_code import CountryCode
    from plaid.model.link_token_account_filters import LinkTokenAccountFilters
    from plaid.model.link_token_create_request_auth import LinkTokenCreateRequestAuth
    from plaid.model.link_token_create_request_deposit_switch import LinkTokenCreateRequestDepositSwitch
    from plaid.model.link_token_create_request_income_verification import LinkTokenCreateRequestIncomeVerification
    from plaid.model.link_token_create_request_payment_initiation import LinkTokenCreateRequestPaymentInitiation
    from plaid.model.link_token_create_request_transfer import LinkTokenCreateRequestTransfer
    from plaid.model.link_token_create_request_update import LinkTokenCreateRequestUpdate
    from plaid.model.link_token_create_request_user import LinkTokenCreateRequestUser
    from plaid.model.link_token_eu_config import LinkTokenEUConfig
    from plaid.model.products import Products
    globals()['CountryCode'] = CountryCode
    globals()['LinkTokenAccountFilters'] = LinkTokenAccountFilters
    globals()['LinkTokenCreateRequestAuth'] = LinkTokenCreateRequestAuth
    globals()['LinkTokenCreateRequestDepositSwitch'] = LinkTokenCreateRequestDepositSwitch
    globals()['LinkTokenCreateRequestIncomeVerification'] = LinkTokenCreateRequestIncomeVerification
    globals()['LinkTokenCreateRequestPaymentInitiation'] = LinkTokenCreateRequestPaymentInitiation
    globals()['LinkTokenCreateRequestTransfer'] = LinkTokenCreateRequestTransfer
    globals()['LinkTokenCreateRequestUpdate'] = LinkTokenCreateRequestUpdate
    globals()['LinkTokenCreateRequestUser'] = LinkTokenCreateRequestUser
    globals()['LinkTokenEUConfig'] = LinkTokenEUConfig
    globals()['Products'] = Products


class LinkTokenCreateRequest(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
        ('country_codes',): {
            'min_items': 1,
        },
    }

    additional_properties_type = None

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'client_name': (str,),  # noqa: E501
            'language': (str,),  # noqa: E501
            'country_codes': ([CountryCode],),  # noqa: E501
            'user': (LinkTokenCreateRequestUser,),  # noqa: E501
            'client_id': (str,),  # noqa: E501
            'secret': (str,),  # noqa: E501
            'products': ([Products],),  # noqa: E501
            'webhook': (str,),  # noqa: E501
            'access_token': (str,),  # noqa: E501
            'link_customization_name': (str,),  # noqa: E501
            'redirect_uri': (str,),  # noqa: E501
            'android_package_name': (str,),  # noqa: E501
            'account_filters': (LinkTokenAccountFilters,),  # noqa: E501
            'eu_config': (LinkTokenEUConfig,),  # noqa: E501
            'institution_id': (str,),  # noqa: E501
            'payment_initiation': (LinkTokenCreateRequestPaymentInitiation,),  # noqa: E501
            'deposit_switch': (LinkTokenCreateRequestDepositSwitch,),  # noqa: E501
            'income_verification': (LinkTokenCreateRequestIncomeVerification,),  # noqa: E501
            'auth': (LinkTokenCreateRequestAuth,),  # noqa: E501
            'transfer': (LinkTokenCreateRequestTransfer,),  # noqa: E501
            'update': (LinkTokenCreateRequestUpdate,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'client_name': 'client_name',  # noqa: E501
        'language': 'language',  # noqa: E501
        'country_codes': 'country_codes',  # noqa: E501
        'user': 'user',  # noqa: E501
        'client_id': 'client_id',  # noqa: E501
        'secret': 'secret',  # noqa: E501
        'products': 'products',  # noqa: E501
        'webhook': 'webhook',  # noqa: E501
        'access_token': 'access_token',  # noqa: E501
        'link_customization_name': 'link_customization_name',  # noqa: E501
        'redirect_uri': 'redirect_uri',  # noqa: E501
        'android_package_name': 'android_package_name',  # noqa: E501
        'account_filters': 'account_filters',  # noqa: E501
        'eu_config': 'eu_config',  # noqa: E501
        'institution_id': 'institution_id',  # noqa: E501
        'payment_initiation': 'payment_initiation',  # noqa: E501
        'deposit_switch': 'deposit_switch',  # noqa: E501
        'income_verification': 'income_verification',  # noqa: E501
        'auth': 'auth',  # noqa: E501
        'transfer': 'transfer',  # noqa: E501
        'update': 'update',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, client_name, language, country_codes, user, *args, **kwargs):  # noqa: E501
        """LinkTokenCreateRequest - a model defined in OpenAPI

        Args:
            client_name (str): The name of your application, as it should be displayed in Link. Maximum length of 30 characters. If a value longer than 30 characters is provided, Link will display \"This Application\" instead.
            language (str): The language that Link should be displayed in.  Supported languages are: - English (`'en'`) - French (`'fr'`) - Spanish (`'es'`) - Dutch (`'nl'`) - German(`'de'`)  When using a Link customization, the language configured here must match the setting in the customization, or the customization will not be applied.
            country_codes ([CountryCode]): Specify an array of Plaid-supported country codes using the ISO-3166-1 alpha-2 country code standard. Institutions from all listed countries will be shown.  Supported country codes are: `US`, `CA`, `DE`, `ES`, `FR`, `GB`, `IE`, `NL`. For a complete mapping of supported products by country, see https://plaid.com/global/.  If Link is launched with multiple country codes, only products that you are enabled for in all countries will be used by Link. Note that while all countries are enabled by default in Sandbox and Development, in Production only US and Canada are enabled by default. To gain access to European institutions in the Production environment, [file a product access Support ticket](https://dashboard.plaid.com/support/new/product-and-development/product-troubleshooting/request-product-access) via the Plaid dashboard. If you initialize with a European country code, your users will see the European consent panel during the Link flow.  If using a Link customization, make sure the country codes in the customization match those specified in `country_codes`. If both `country_codes` and a Link customization are used, the value in `country_codes` may override the value in the customization.  If using the Auth features Instant Match, Same-day Micro-deposits, or Automated Micro-deposits, `country_codes` must be set to `['US']`.
            user (LinkTokenCreateRequestUser):

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            client_id (str): Your Plaid API `client_id`. The `client_id` is required and may be provided either in the `PLAID-CLIENT-ID` header or as part of a request body.. [optional]  # noqa: E501
            secret (str): Your Plaid API `secret`. The `secret` is required and may be provided either in the `PLAID-SECRET` header or as part of a request body.. [optional]  # noqa: E501
            products ([Products]): List of Plaid product(s) you wish to use. If launching Link in update mode, should be omitted; required otherwise. Valid products are:  `transactions`, `auth`, `identity`, `assets`, `investments`, `liabilities`, `payment_initiation`, `deposit_switch`, `income_verification`, `transfer`  `balance` is *not* a valid value, the Balance product does not require explicit initalization and will automatically be initialized when any other product is initialized.  Only institutions that support *all* requested products will be shown in Link; to maximize the number of institutions listed, it is recommended to initialize Link with the minimal product set required for your use case. Additional products can be added after Link initialization by calling the relevant endpoints. For details and exceptions, see [Choosing when to initialize products](https://plaid.com/docs/link/best-practices/#choosing-when-to-initialize-products).  Note that, unless you have opted to disable Instant Match support, institutions that support Instant Match will also be shown in Link if `auth` is specified as a product, even though these institutions do not contain `auth` in their product array.  In Production, you will be billed for each product that you specify when initializing Link. Note that a product cannot be removed from an Item once the Item has been initialized with that product. To stop billing on an Item for subscription-based products, such as Liabilities, Investments, and Transactions, remove the Item via `/item/remove`.. [optional]  # noqa: E501
            webhook (str): The destination URL to which any webhooks should be sent.. [optional]  # noqa: E501
            access_token (str): The `access_token` associated with the Item to update, used when updating or modifying an existing `access_token`. Used when launching Link in update mode, when completing the Same-day (manual) Micro-deposit flow, or (optionally) when initializing Link as part of the Payment Initiation (UK and Europe) flow.. [optional]  # noqa: E501
            link_customization_name (str): The name of the Link customization from the Plaid Dashboard to be applied to Link. If not specified, the `default` customization will be used. When using a Link customization, the language in the customization must match the language selected via the `language` parameter, and the countries in the customization should match the country codes selected via `country_codes`.. [optional]  # noqa: E501
            redirect_uri (str): A URI indicating the destination where a user should be forwarded after completing the Link flow; used to support OAuth authentication flows when launching Link in the browser or via a webview. The `redirect_uri` should not contain any query parameters. When used in Production or Development, must be an https URI. To specify any subdomain, use `*` as a wildcard character, e.g. `https://*.example.com/oauth.html`. If `android_package_name` is specified, this field should be left blank.  Note that any redirect URI must also be added to the Allowed redirect URIs list in the [developer dashboard](https://dashboard.plaid.com/team/api).. [optional]  # noqa: E501
            android_package_name (str): The name of your app's Android package. Required if using the `link_token` to initialize Link on Android. When creating a `link_token` for initializing Link on other platforms, this field must be left blank. Any package name specified here must also be added to the Allowed Android package names setting on the [developer dashboard](https://dashboard.plaid.com/team/api). . [optional]  # noqa: E501
            account_filters (LinkTokenAccountFilters): [optional]  # noqa: E501
            eu_config (LinkTokenEUConfig): [optional]  # noqa: E501
            institution_id (str): Used for certain Europe-only configurations, as well as certain legacy use cases in other regions.. [optional]  # noqa: E501
            payment_initiation (LinkTokenCreateRequestPaymentInitiation): [optional]  # noqa: E501
            deposit_switch (LinkTokenCreateRequestDepositSwitch): [optional]  # noqa: E501
            income_verification (LinkTokenCreateRequestIncomeVerification): [optional]  # noqa: E501
            auth (LinkTokenCreateRequestAuth): [optional]  # noqa: E501
            transfer (LinkTokenCreateRequestTransfer): [optional]  # noqa: E501
            update (LinkTokenCreateRequestUpdate): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.client_name = client_name
        self.language = language
        self.country_codes = country_codes
        self.user = user
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)


class DepositSwitchGetResponse(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('state',): {
            'INITIALIZED': "initialized",
            'PROCESSING': "processing",
            'COMPLETED': "completed",
            'ERROR': "error",
        },
        ('switch_method',): {
            'None': None,
            'INSTANT': "instant",
            'MAIL': "mail",
            'PDF': "pdf",
            'NULL': "null",
        },
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            'deposit_switch_id': (str,),  # noqa: E501
            'target_account_id': (str, none_type,),  # noqa: E501
            'target_item_id': (str, none_type,),  # noqa: E501
            'state': (str,),  # noqa: E501
            'account_has_multiple_allocations': (bool, none_type,),  # noqa: E501
            'is_allocated_remainder': (bool, none_type,),  # noqa: E501
            'percent_allocated': (float, none_type,),  # noqa: E501
            'amount_allocated': (float, none_type,),  # noqa: E501
            'date_created': (date,),  # noqa: E501
            'date_completed': (date, none_type,),  # noqa: E501
            'request_id': (str,),  # noqa: E501
            'switch_method': (str, none_type,),  # noqa: E501
            'employer_name': (str, none_type,),  # noqa: E501
            'employer_id': (str, none_type,),  # noqa: E501
            'institution_name': (str, none_type,),  # noqa: E501
            'institution_id': (str, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'deposit_switch_id': 'deposit_switch_id',  # noqa: E501
        'target_account_id': 'target_account_id',  # noqa: E501
        'target_item_id': 'target_item_id',  # noqa: E501
        'state': 'state',  # noqa: E501
        'account_has_multiple_allocations': 'account_has_multiple_allocations',  # noqa: E501
        'is_allocated_remainder': 'is_allocated_remainder',  # noqa: E501
        'percent_allocated': 'percent_allocated',  # noqa: E501
        'amount_allocated': 'amount_allocated',  # noqa: E501
        'date_created': 'date_created',  # noqa: E501
        'date_completed': 'date_completed',  # noqa: E501
        'request_id': 'request_id',  # noqa: E501
        'switch_method': 'switch_method',  # noqa: E501
        'employer_name': 'employer_name',  # noqa: E501
        'employer_id': 'employer_id',  # noqa: E501
        'institution_name': 'institution_name',  # noqa: E501
        'institution_id': 'institution_id',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, deposit_switch_id, target_account_id, target_item_id, state, account_has_multiple_allocations, is_allocated_remainder, percent_allocated, amount_allocated, date_created, date_completed, request_id, *args, **kwargs):  # noqa: E501
        """DepositSwitchGetResponse - a model defined in OpenAPI

        Args:
            deposit_switch_id (str): The ID of the deposit switch.
            target_account_id (str, none_type): The ID of the bank account the direct deposit was switched to.
            target_item_id (str, none_type): The ID of the Item the direct deposit was switched to.
            state (str):  The state, or status, of the deposit switch.  - `initialized` – The deposit switch has been initialized with the user entering the information required to submit the deposit switch request.  - `processing` – The deposit switch request has been submitted and is being processed.  - `completed` – The user's employer has fulfilled the deposit switch request.  - `error` – There was an error processing the deposit switch request.
            account_has_multiple_allocations (bool, none_type): When `true`, user’s direct deposit goes to multiple banks. When false, user’s direct deposit only goes to the target account. Always `null` if the deposit switch has not been completed.
            is_allocated_remainder (bool, none_type): When `true`, the target account is allocated the remainder of direct deposit after all other allocations have been deducted. When `false`, user’s direct deposit is allocated as a percent or amount. Always `null` if the deposit switch has not been completed.
            percent_allocated (float, none_type): The percentage of direct deposit allocated to the target account. Always `null` if the target account is not allocated a percentage or if the deposit switch has not been completed or if `is_allocated_remainder` is true.
            amount_allocated (float, none_type): The dollar amount of direct deposit allocated to the target account. Always `null` if the target account is not allocated an amount or if the deposit switch has not been completed.
            date_created (date): [ISO 8601](https://wikipedia.org/wiki/ISO_8601) date the deposit switch was created. 
            date_completed (date, none_type): [ISO 8601](https://wikipedia.org/wiki/ISO_8601) date the deposit switch was completed. Always `null` if the deposit switch has not been completed. 
            request_id (str): A unique identifier for the request, which can be used for troubleshooting. This identifier, like all Plaid identifiers, is case sensitive.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            switch_method (str, none_type): The method used to make the deposit switch.  - `instant` – User instantly switched their direct deposit to a new or existing bank account by connecting their payroll or employer account.  - `mail` – User requested that Plaid contact their employer by mail to make the direct deposit switch.  - `pdf` – User generated a PDF or email to be sent to their employer with the information necessary to make the deposit switch.'. [optional]  # noqa: E501
            employer_name (str, none_type): The name of the employer selected by the user. If the user did not select an employer, the value returned is `null`.. [optional]  # noqa: E501
            employer_id (str, none_type): The ID of the employer selected by the user. If the user did not select an employer, the value returned is `null`.. [optional]  # noqa: E501
            institution_name (str, none_type): The name of the institution selected by the user. If the user did not select an institution, the value returned is `null`.. [optional]  # noqa: E501
            institution_id (str, none_type): The ID of the institution selected by the user. If the user did not select an institution, the value returned is `null`.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.deposit_switch_id = deposit_switch_id
        self.target_account_id = target_account_id
        self.target_item_id = target_item_id
        self.state = state
        self.account_has_multiple_allocations = account_has_multiple_allocations
        self.is_allocated_remainder = is_allocated_remainder
        self.percent_allocated = percent_allocated
        self.amount_allocated = amount_allocated
        self.date_created = date_created
        self.date_completed = date_completed
        self.request_id = request_id
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



import time
import datetime as dt
import plaid
import json
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.sandbox_public_token_create_request_options import SandboxPublicTokenCreateRequestOptions
from plaid.model.sandbox_public_token_create_request_options_transactions import SandboxPublicTokenCreateRequestOptionsTransactions
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.investments_transactions_get_request import InvestmentsTransactionsGetRequest
from plaid.model.investments_transactions_get_request_options import InvestmentsTransactionsGetRequestOptions
from plaid.model.item_remove_request import ItemRemoveRequest


from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
)

access_token = None

START_DATE = dt.date(2020, 1, 1)
END_DATE = dt.date(2021, 1, 1)


def setup_module(module):
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('investments')],
        options=SandboxPublicTokenCreateRequestOptions(
            transactions=SandboxPublicTokenCreateRequestOptionsTransactions(
                start_date=START_DATE,
                end_date=END_DATE
            )
        )
    )
    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )
    exchange_response = client.item_public_token_exchange(exchange_request)

    global access_token
    access_token = exchange_response['access_token']


def get_investment_transactions_with_retries(client,
                                             access_token,
                                             start_date,
                                             end_date,
                                             account_ids=None,
                                             count=None,
                                             offset=None,
                                             num_retries=5):
    response = None
    for i in range(num_retries):
        try:
            options = InvestmentsTransactionsGetRequestOptions()

            if count is not None:
                options.count = count
            if offset is not None:
                options.offset = offset
            if account_ids is not None:
                options.account_ids = account_ids

            request = InvestmentsTransactionsGetRequest(
                access_token=access_token,
                start_date=start_date,
                end_date=end_date,
                options=options
            )
            response = \
                client.investments_transactions_get(request)
        except plaid.ApiException as e:
            response = json.loads(e.body)
            if response.code == 'PRODUCT_NOT_READY':
                time.sleep(5)
                continue
            else:
                raise e
        break
    return response


def teardown_module(module):
    client = create_client()
    ir_request = ItemRemoveRequest(
        access_token=access_token
    )
    client.item_remove(ir_request)


def test_get():
    client = create_client()

    response = get_investment_transactions_with_retries(client,
                                                        access_token,
                                                        START_DATE,
                                                        END_DATE,
                                                        num_retries=5)
    assert response['item'] is not None
    assert response['accounts'] is not None
    assert response['securities'] is not None
    assert response['investment_transactions'] is not None
    assert response['total_investment_transactions'] is not None

    # get transactions for selected accounts
    account_id = response['accounts'][0]['account_id']
    response = \
        get_investment_transactions_with_retries(client,
                                                 access_token,
                                                 START_DATE,
                                                 END_DATE,
                                                 account_ids=[account_id],
                                                 num_retries=5)
    assert response['investment_transactions'] is not None


def test_get_with_options():
    client = create_client()
    response = get_investment_transactions_with_retries(client,
                                                        access_token,
                                                        START_DATE,
                                                        END_DATE,
                                                        count=2,
                                                        offset=1)
    assert len(response['investment_transactions']) == 2



import json
import plaid
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.item_remove_request import ItemRemoveRequest
from plaid.model.auth_get_request import AuthGetRequest
from plaid.model.auth_get_request_options import AuthGetRequestOptions

from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
)

access_token = None

def setup_module(module):
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('auth')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    global access_token
    access_token = exchange_response['access_token']


def teardown_module(module):
    client = create_client()
    ir_request = ItemRemoveRequest(
        access_token=access_token
    )
    client.item_remove(ir_request)


def test_get():
    client = create_client()

    ag_request = AuthGetRequest(
        access_token=access_token
    )
    # get auth for all accounts
    response = client.auth_get(ag_request)
    assert response['accounts'] is not None
    assert response['numbers'] is not None

    # get auth for selected accounts
    account_id = response['accounts'][0]['account_id']
    ag_request = AuthGetRequest(
        access_token=access_token,
        options=AuthGetRequestOptions(account_ids=[account_id])
    )
    response = client.auth_get(ag_request)
    for key in [
        'eft',
        'ach',
        'international',
        'bacs',
    ]:
        assert key in response['numbers']


def test_auth_error():
    client = create_client()

    corrupted_access_token = access_token[:-1] + chr(ord(access_token[-1]) + 1)

    try:
        ag_request = AuthGetRequest(
            access_token=corrupted_access_token,
        )
        response = client.auth_get(ag_request)
    except plaid.ApiException as e:
        response = json.loads(e.body)
        assert response['error_code'] == 'INVALID_ACCESS_TOKEN'


'''
Client.Item.* tests.

Note that when using many of these methods in production, it would be necessary
to handle error cases (e.g. ``INVALID_CREDENTIAL``). However, we don't do that
here since any errors will automatically be marked as failures by the test
runner.
'''

import json
from contextlib import contextmanager

import plaid
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.item_remove_request import ItemRemoveRequest
from plaid.model.item_get_request import ItemGetRequest
from plaid.model.item_import_request import ItemImportRequest
from plaid.model.item_import_request_user_auth import ItemImportRequestUserAuth
from plaid.model.sandbox_item_fire_webhook_request import SandboxItemFireWebhookRequest
from plaid.model.item_access_token_invalidate_request import ItemAccessTokenInvalidateRequest
from plaid.model.item_webhook_update_request import ItemWebhookUpdateRequest
from plaid.model.sandbox_public_token_create_request_options import SandboxPublicTokenCreateRequestOptions


from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
)

# Ensure that any items created are also removed


@contextmanager
def ensure_item_removed(access_token):
    try:
        yield
    finally:
        request = ItemRemoveRequest(
            access_token=access_token
        )
        client = create_client()
        client.item_remove(request)


def test_get():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    with ensure_item_removed(exchange_response['access_token']):
        get_request = ItemGetRequest(
            access_token=exchange_response['access_token']
        )
        get_response = client.item_get(get_request)
        assert get_response['item'] is not None


def test_remove():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)
    ir_request = ItemRemoveRequest(
        access_token=exchange_response['access_token']
    )

    remove_response = client.item_remove(ir_request)
    assert remove_response['request_id']

    try:
        ir_request = ItemRemoveRequest(
            access_token=exchange_response['access_token']
        )
        client.item_remove(ir_request)
    except plaid.ApiException as e:
        response = json.loads(e.body)
        assert response['error_code'] == 'INVALID_ACCESS_TOKEN'


def test_import():
    client = create_client()
    at_request = ItemImportRequest(
        products=[Products('identity'), Products('auth')],
        user_auth=ItemImportRequestUserAuth(
            user_id='user_good',
            auth_token='pass_good'
        )
    )
    at_response = client.item_import(at_request)
    assert at_response['access_token'] is not None


def test_public_token():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)
    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    with ensure_item_removed(exchange_response['access_token']):
        assert pt_response['public_token'] is not None
        assert exchange_response['access_token'] is not None


def test_sandbox_public_token():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)
    assert pt_response['public_token'] is not None

    # public token -> access token
    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)
    assert exchange_response['access_token'] is not None


def test_sandbox_fire_webhook():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')],
        options=SandboxPublicTokenCreateRequestOptions(
            webhook='https://plaid.com/foo/bar/hook'
        )
    )

    pt_response = client.sandbox_public_token_create(pt_request)
    assert pt_response['public_token'] is not None

    # public token -> access token
    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)
    assert exchange_response['access_token'] is not None

    # fire webhook
    fire_request = SandboxItemFireWebhookRequest(
        access_token=exchange_response['access_token'],
        webhook_code='DEFAULT_UPDATE'
    )

    fire_webhook_response = client.sandbox_item_fire_webhook(fire_request)
    assert fire_webhook_response['webhook_fired'] is True


def test_access_token_invalidate():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)
    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    try:
        invalidate_request = ItemAccessTokenInvalidateRequest(
            access_token=exchange_response['access_token']
        )
        invalidate_response = client.item_access_token_invalidate(
            invalidate_request)
        with ensure_item_removed(invalidate_response['new_access_token']):
            assert invalidate_response['new_access_token'] is not None
    except Exception:
        with ensure_item_removed(exchange_response['access_token']):
            raise


def test_webhook_update():
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')]
    )

    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    with ensure_item_removed(exchange_response['access_token']):
        webhook_request = ItemWebhookUpdateRequest(
            access_token=exchange_response['access_token'],
            webhook='https://plaid.com/webhook-test'
        )
        webhook_response = client.item_webhook_update(webhook_request)
        assert (webhook_response['item']['webhook'] ==
                'https://plaid.com/webhook-test')


"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)

def lazy_import():
    from plaid.model.pslf_status import PSLFStatus
    from plaid.model.servicer_address_data import ServicerAddressData
    from plaid.model.student_loan_status import StudentLoanStatus
    from plaid.model.student_repayment_plan import StudentRepaymentPlan
    globals()['PSLFStatus'] = PSLFStatus
    globals()['ServicerAddressData'] = ServicerAddressData
    globals()['StudentLoanStatus'] = StudentLoanStatus
    globals()['StudentRepaymentPlan'] = StudentRepaymentPlan


class StudentLoan(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'account_id': (str, none_type,),  # noqa: E501
            'account_number': (str, none_type,),  # noqa: E501
            'disbursement_dates': ([date], none_type,),  # noqa: E501
            'expected_payoff_date': (date, none_type,),  # noqa: E501
            'guarantor': (str, none_type,),  # noqa: E501
            'interest_rate_percentage': (float,),  # noqa: E501
            'is_overdue': (bool, none_type,),  # noqa: E501
            'last_payment_amount': (float, none_type,),  # noqa: E501
            'last_payment_date': (date, none_type,),  # noqa: E501
            'last_statement_issue_date': (date, none_type,),  # noqa: E501
            'loan_name': (str, none_type,),  # noqa: E501
            'loan_status': (StudentLoanStatus,),  # noqa: E501
            'minimum_payment_amount': (float, none_type,),  # noqa: E501
            'next_payment_due_date': (date, none_type,),  # noqa: E501
            'origination_date': (date, none_type,),  # noqa: E501
            'origination_principal_amount': (float, none_type,),  # noqa: E501
            'outstanding_interest_amount': (float, none_type,),  # noqa: E501
            'payment_reference_number': (str, none_type,),  # noqa: E501
            'pslf_status': (PSLFStatus,),  # noqa: E501
            'repayment_plan': (StudentRepaymentPlan,),  # noqa: E501
            'sequence_number': (str, none_type,),  # noqa: E501
            'servicer_address': (ServicerAddressData,),  # noqa: E501
            'ytd_interest_paid': (float, none_type,),  # noqa: E501
            'ytd_principal_paid': (float, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'account_id': 'account_id',  # noqa: E501
        'account_number': 'account_number',  # noqa: E501
        'disbursement_dates': 'disbursement_dates',  # noqa: E501
        'expected_payoff_date': 'expected_payoff_date',  # noqa: E501
        'guarantor': 'guarantor',  # noqa: E501
        'interest_rate_percentage': 'interest_rate_percentage',  # noqa: E501
        'is_overdue': 'is_overdue',  # noqa: E501
        'last_payment_amount': 'last_payment_amount',  # noqa: E501
        'last_payment_date': 'last_payment_date',  # noqa: E501
        'last_statement_issue_date': 'last_statement_issue_date',  # noqa: E501
        'loan_name': 'loan_name',  # noqa: E501
        'loan_status': 'loan_status',  # noqa: E501
        'minimum_payment_amount': 'minimum_payment_amount',  # noqa: E501
        'next_payment_due_date': 'next_payment_due_date',  # noqa: E501
        'origination_date': 'origination_date',  # noqa: E501
        'origination_principal_amount': 'origination_principal_amount',  # noqa: E501
        'outstanding_interest_amount': 'outstanding_interest_amount',  # noqa: E501
        'payment_reference_number': 'payment_reference_number',  # noqa: E501
        'pslf_status': 'pslf_status',  # noqa: E501
        'repayment_plan': 'repayment_plan',  # noqa: E501
        'sequence_number': 'sequence_number',  # noqa: E501
        'servicer_address': 'servicer_address',  # noqa: E501
        'ytd_interest_paid': 'ytd_interest_paid',  # noqa: E501
        'ytd_principal_paid': 'ytd_principal_paid',  # noqa: E501
    }

    _composed_schemas = {}

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, account_id, account_number, disbursement_dates, expected_payoff_date, guarantor, interest_rate_percentage, is_overdue, last_payment_amount, last_payment_date, last_statement_issue_date, loan_name, loan_status, minimum_payment_amount, next_payment_due_date, origination_date, origination_principal_amount, outstanding_interest_amount, payment_reference_number, pslf_status, repayment_plan, sequence_number, servicer_address, ytd_interest_paid, ytd_principal_paid, *args, **kwargs):  # noqa: E501
        """StudentLoan - a model defined in OpenAPI

        Args:
            account_id (str, none_type): The ID of the account that this liability belongs to.
            account_number (str, none_type): The account number of the loan. For some institutions, this may be a masked version of the number (e.g., the last 4 digits instead of the entire number).
            disbursement_dates ([date], none_type): The dates on which loaned funds were disbursed or will be disbursed. These are often in the past. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD).
            expected_payoff_date (date, none_type): The date when the student loan is expected to be paid off. Availability for this field is limited. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD).
            guarantor (str, none_type): The guarantor of the student loan.
            interest_rate_percentage (float): The interest rate on the loan as a percentage.
            is_overdue (bool, none_type): `true` if a payment is currently overdue. Availability for this field is limited.
            last_payment_amount (float, none_type): The amount of the last payment.
            last_payment_date (date, none_type): The date of the last payment. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD).
            last_statement_issue_date (date, none_type): The date of the last statement. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD).
            loan_name (str, none_type): The type of loan, e.g., \"Consolidation Loans\".
            loan_status (StudentLoanStatus):
            minimum_payment_amount (float, none_type): The minimum payment due for the next billing cycle. There are some exceptions: Some institutions require a minimum payment across all loans associated with an account number. Our API presents that same minimum payment amount on each loan. The institutions that do this are: Great Lakes ( `ins_116861`), Firstmark (`ins_116295`), Commonbond Firstmark Services (`ins_116950`), Nelnet (`ins_116528`), EdFinancial Services (`ins_116304`), Granite State (`ins_116308`), and Oklahoma Student Loan Authority (`ins_116945`). Firstmark (`ins_116295` ) will display as $0 if there is an autopay program in effect.
            next_payment_due_date (date, none_type): The due date for the next payment. The due date is `null` if a payment is not expected. A payment is not expected if `loan_status.type` is `deferment`, `in_school`, `consolidated`, `paid in full`, or `transferred`. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD).
            origination_date (date, none_type): The date on which the loan was initially lent. Dates are returned in an [ISO 8601](https://wikipedia.org/wiki/ISO_8601) format (YYYY-MM-DD). 
            origination_principal_amount (float, none_type): The original principal balance of the loan.
            outstanding_interest_amount (float, none_type): The total dollar amount of the accrued interest balance. For Sallie Mae ( `ins_116944`), this amount is included in the current balance of the loan, so this field will return as `null`.
            payment_reference_number (str, none_type): The relevant account number that should be used to reference this loan for payments. In the majority of cases, `payment_reference_number` will match a`ccount_number,` but in some institutions, such as Great Lakes (`ins_116861`), it will be different.
            pslf_status (PSLFStatus):
            repayment_plan (StudentRepaymentPlan):
            sequence_number (str, none_type): The sequence number of the student loan. Heartland ECSI (`ins_116948`) does not make this field available.
            servicer_address (ServicerAddressData):
            ytd_interest_paid (float, none_type): The year to date (YTD) interest paid. Availability for this field is limited.
            ytd_principal_paid (float, none_type): The year to date (YTD) principal paid. Availability for this field is limited.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.account_id = account_id
        self.account_number = account_number
        self.disbursement_dates = disbursement_dates
        self.expected_payoff_date = expected_payoff_date
        self.guarantor = guarantor
        self.interest_rate_percentage = interest_rate_percentage
        self.is_overdue = is_overdue
        self.last_payment_amount = last_payment_amount
        self.last_payment_date = last_payment_date
        self.last_statement_issue_date = last_statement_issue_date
        self.loan_name = loan_name
        self.loan_status = loan_status
        self.minimum_payment_amount = minimum_payment_amount
        self.next_payment_due_date = next_payment_due_date
        self.origination_date = origination_date
        self.origination_principal_amount = origination_principal_amount
        self.outstanding_interest_amount = outstanding_interest_amount
        self.payment_reference_number = payment_reference_number
        self.pslf_status = pslf_status
        self.repayment_plan = repayment_plan
        self.sequence_number = sequence_number
        self.servicer_address = servicer_address
        self.ytd_interest_paid = ytd_interest_paid
        self.ytd_principal_paid = ytd_principal_paid
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)



'''
Client.Processor.* tests.
'''
from tests.integration.util import (
    create_client
)
import pytest
import plaid
import json
from plaid.model.processor_token_create_request import ProcessorTokenCreateRequest
from plaid.model.processor_stripe_bank_account_token_create_request import ProcessorStripeBankAccountTokenCreateRequest


def test_stripe_processor_token():
    client = create_client()
    # Just test the failure case - behavior here depends on the API keys used
    with pytest.raises(plaid.ApiException) as e:
        request = ProcessorStripeBankAccountTokenCreateRequest(
            access_token='fakeAccessToken',
            account_id='fakeAccountId',
        )
        client.processor_stripe_bank_account_token_create(request)
        response = json.loads(e.body)
        assert response['error_code'] == 'INVALID_INPUT'


def test_dwolla_processor_token():
    client = create_client()
    # Just test the failure case - behavior here depends on the API keys used
    with pytest.raises(plaid.ApiException) as e:
        request = ProcessorTokenCreateRequest(
            access_token='fakeAccessToken',
            account_id='fakeAccountId',
            processor='dwolla'
        )
        client.processor_token_create(request)
        response = json.loads(e.body)
        assert response['error_code'] == 'INVALID_INPUT'



import time
import plaid
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.accounts_get_request import AccountsGetRequest
from plaid.model.bank_transfer_create_request import BankTransferCreateRequest
from plaid.model.bank_transfer_network import BankTransferNetwork
from plaid.model.bank_transfer_idempotency_key import BankTransferIdempotencyKey
from plaid.model.bank_transfer_type import BankTransferType
from plaid.model.bank_transfer_user import BankTransferUser
from plaid.model.bank_transfer_event_list_request import BankTransferEventListRequest
from plaid.model.bank_transfer_cancel_request import BankTransferCancelRequest
from plaid.model.bank_transfer_idempotency_key import BankTransferIdempotencyKey
from plaid.model.sandbox_bank_transfer_simulate_request import SandboxBankTransferSimulateRequest
from plaid.model.bank_transfer_list_request import BankTransferListRequest
from plaid.model.bank_transfer_event_sync_request import BankTransferEventSyncRequest
from plaid.model.bank_transfer_get_request import BankTransferGetRequest
from plaid.model.bank_transfer_balance_get_request import BankTransferBalanceGetRequest
from plaid.model.bank_transfer_migrate_account_request import BankTransferMigrateAccountRequest
from plaid.model.account_type import AccountType
from plaid.model.ach_class import ACHClass

from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION
)

from random import random
from random import seed

access_token = None
account_id = None
num_retries = 10


def setup_module(module):
    client = create_client()

    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('auth')]
    )
    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )
    exchange_response = client.item_public_token_exchange(exchange_request)

    global access_token
    access_token = exchange_response['access_token']

    accounts_request = AccountsGetRequest(
        access_token=access_token
    )
    accounts_response = client.accounts_get(accounts_request)
    account = next(
        acct for acct in accounts_response['accounts'] if acct['type'] == AccountType('depository'))

    global account_id
    account_id = account['account_id']
    seed()


def create_bank_transfer(client):
    bt_request = BankTransferCreateRequest(
        idempotency_key=BankTransferIdempotencyKey(str(random())),
        access_token=access_token,
        account_id=account_id,
        type=BankTransferType('credit'),
        network=BankTransferNetwork('ach'),
        amount='1.00',
        iso_currency_code='USD',
        description='test',
        user=BankTransferUser(legal_name='Firstname Lastname'),
        ach_class=ACHClass('ppd'),
        custom_tag='',
    )
    bt_response = client.bank_transfer_create(bt_request)
    assert bt_response is not None
    return bt_response['bank_transfer']


def poll_for_response(checks):
    success = False
    for i in range(num_retries):
        try:
            checks()
            success = True
            break
        except AssertionError:
            time.sleep(0.5)
            continue
    assert success


def test_create():
    client = create_client()
    bt = create_bank_transfer(client)
    assert bt is not None
    assert bt['id'] is not None


def test_cancel_success():
    client = create_client()
    bt = create_bank_transfer(client)

    def checks():
        cancel_request = BankTransferCancelRequest(
            bank_transfer_id=bt['id']
        )
        cancel_response = client.bank_transfer_cancel(cancel_request)
        assert cancel_response is not None
    poll_for_response(checks)


def test_cancel_failure():
    client = create_client()
    bt = create_bank_transfer(client)

    simulate_request = SandboxBankTransferSimulateRequest(
        bank_transfer_id=bt['id'],
        event_type='posted'
    )
    simulate_response = client.sandbox_bank_transfer_simulate(simulate_request)
    assert simulate_response is not None
    # allow event simulation to process so we don't inadvertently succeed
    time.sleep(0.1)
    # assert that a BankTransferError is thrown
    try:
        cancel_request = BankTransferCancelRequest(
            bank_transfer_id=bt['id']
        )
        client.bank_transfer_cancel(cancel_request)
        # cancel should not succeed
        assert False
    except plaid.ApiException:
        assert True
    except:
        assert False


def test_list_events():
    client = create_client()
    bt = create_bank_transfer(client)
    bt_id = bt['id']
    simulate_request = SandboxBankTransferSimulateRequest(
        bank_transfer_id=bt_id,
        event_type='posted'
    )
    simulate_response = client.sandbox_bank_transfer_simulate(simulate_request)
    assert simulate_response is not None

    def checks():
        list_events_request = BankTransferEventListRequest(
            bank_transfer_id=bt_id
        )
        list_events_response = client.bank_transfer_event_list(
            list_events_request)
        assert list_events_response is not None
        assert list_events_response['bank_transfer_events'] is not None
        assert len(list_events_response['bank_transfer_events']) >= 1
    poll_for_response(checks)


def test_sync_events():
    client = create_client()
    bt = create_bank_transfer(client)

    def checks():
        sync_request = BankTransferEventSyncRequest(
            after_id=0
        )
        sync_response = client.bank_transfer_event_sync(sync_request)
        assert sync_response is not None
        assert sync_response['bank_transfer_events'] is not None
        assert len(sync_response['bank_transfer_events']) > 0
    poll_for_response(checks)


def test_get():
    client = create_client()
    bt = create_bank_transfer(client)

    def checks():
        get_request = BankTransferGetRequest(
            bank_transfer_id=bt['id']
        )
        get_response = client.bank_transfer_get(get_request)
        assert bt == get_response['bank_transfer']
    poll_for_response(checks)


def test_list():
    client = create_client()
    bt = create_bank_transfer(client)

    def checks():
        list_request = BankTransferListRequest(
            count=1
        )
        list_response = client.bank_transfer_list(list_request)
        assert list_response is not None
        assert len(list_response['bank_transfers']) == 1
    poll_for_response(checks)


def test_balance_get():
    client = create_client()
    balance_request = BankTransferBalanceGetRequest()
    balance_response = client.bank_transfer_balance_get(balance_request)
    assert balance_response is not None
    assert balance_response['balance'] is not None
    assert balance_response['balance']['available'] is not None


def test_migrate_account():
    client = create_client()
    migrate_request = BankTransferMigrateAccountRequest(
        account_number = '100000000',
        routing_number = '121122676',
        account_type = 'checking'
    )
    migrate_response = client.bank_transfer_migrate_account(migrate_request)
    assert migrate_response is not None
    assert migrate_response['access_token'] is not None



'''Shared objects for integration testing.'''

import os
import plaid
from plaid.api import plaid_api

client_id = os.environ['CLIENT_ID']
secret = os.environ['SECRET']

def create_client():
    '''Create a new client for testing.'''
    configuration = plaid.Configuration(
        host=plaid.Environment.Sandbox,
        api_key={
            'clientId': client_id,
            'secret': secret,
            'plaidVersion': '2020-09-14'
        }
    )


    api_client = plaid.ApiClient(configuration)
    return plaid_api.PlaidApi(api_client)

SANDBOX_INSTITUTION = 'ins_109508'
SANDBOX_INSTITUTION_NAME = 'First Platypus Bank'

SANDBOX_INSTITUTIONS = [
    'ins_109508',
    'ins_109509',
    'ins_109510',
    'ins_109511',
    'ins_109512',
]

WEBHOOK_VERIFICATION_KEY_ID = '6c5516e1-92dc-479e-a8ff-5a51992e0001'



"""
    The Plaid API

    The Plaid REST API. Please see https://plaid.com/docs/api for more details.  # noqa: E501
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from plaid.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)

def lazy_import():
    from plaid.model.account_assets_all_of import AccountAssetsAllOf
    from plaid.model.account_balance import AccountBalance
    from plaid.model.account_base import AccountBase
    from plaid.model.account_subtype import AccountSubtype
    from plaid.model.account_type import AccountType
    from plaid.model.asset_report_transaction import AssetReportTransaction
    from plaid.model.historical_balance import HistoricalBalance
    from plaid.model.owner import Owner
    globals()['AccountAssetsAllOf'] = AccountAssetsAllOf
    globals()['AccountBalance'] = AccountBalance
    globals()['AccountBase'] = AccountBase
    globals()['AccountSubtype'] = AccountSubtype
    globals()['AccountType'] = AccountType
    globals()['AssetReportTransaction'] = AssetReportTransaction
    globals()['HistoricalBalance'] = HistoricalBalance
    globals()['Owner'] = Owner


class AccountAssets(ModelComposed):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('verification_status',): {
            'AUTOMATICALLY_VERIFIED': "automatically_verified",
            'PENDING_AUTOMATIC_VERIFICATION': "pending_automatic_verification",
            'PENDING_MANUAL_VERIFICATION': "pending_manual_verification",
            'MANUALLY_VERIFIED': "manually_verified",
            'VERIFICATION_EXPIRED': "verification_expired",
            'VERIFICATION_FAILED': "verification_failed",
        },
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'account_id': (str,),  # noqa: E501
            'balances': (AccountBalance,),  # noqa: E501
            'mask': (str, none_type,),  # noqa: E501
            'name': (str,),  # noqa: E501
            'official_name': (str, none_type,),  # noqa: E501
            'type': (AccountType,),  # noqa: E501
            'subtype': (AccountSubtype,),  # noqa: E501
            'days_available': (float,),  # noqa: E501
            'transactions': ([AssetReportTransaction],),  # noqa: E501
            'owners': ([Owner],),  # noqa: E501
            'historical_balances': ([HistoricalBalance],),  # noqa: E501
            'verification_status': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'account_id': 'account_id',  # noqa: E501
        'balances': 'balances',  # noqa: E501
        'mask': 'mask',  # noqa: E501
        'name': 'name',  # noqa: E501
        'official_name': 'official_name',  # noqa: E501
        'type': 'type',  # noqa: E501
        'subtype': 'subtype',  # noqa: E501
        'days_available': 'days_available',  # noqa: E501
        'transactions': 'transactions',  # noqa: E501
        'owners': 'owners',  # noqa: E501
        'historical_balances': 'historical_balances',  # noqa: E501
        'verification_status': 'verification_status',  # noqa: E501
    }

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
        '_composed_instances',
        '_var_name_to_model_instances',
        '_additional_properties_model_instances',
    ])

    @convert_js_args_to_python_args
    def __init__(self, account_id, balances, mask, name, official_name, type, subtype, days_available, transactions, owners, historical_balances, *args, **kwargs):  # noqa: E501
        """AccountAssets - a model defined in OpenAPI

        Args:
            account_id (str): Plaid’s unique identifier for the account. This value will not change unless Plaid can't reconcile the account with the data returned by the financial institution. This may occur, for example, when the name of the account changes. If this happens a new `account_id` will be assigned to the account.  The `account_id` can also change if the `access_token` is deleted and the same credentials that were used to generate that `access_token` are used to generate a new `access_token` on a later date. In that case, the new `account_id` will be different from the old `account_id`.  If an account with a specific `account_id` disappears instead of changing, the account is likely closed. Closed accounts are not returned by the Plaid API.  Like all Plaid identifiers, the `account_id` is case sensitive.
            balances (AccountBalance):
            mask (str, none_type): The last 2-4 alphanumeric characters of an account's official account number. Note that the mask may be non-unique between an Item's accounts, and it may also not match the mask that the bank displays to the user.
            name (str): The name of the account, either assigned by the user or by the financial institution itself
            official_name (str, none_type): The official name of the account as given by the financial institution
            type (AccountType):
            subtype (AccountSubtype):
            days_available (float): The duration of transaction history available for this Item, typically defined as the time since the date of the earliest transaction in that account. Only returned by Assets endpoints.
            transactions ([AssetReportTransaction]): Transaction history associated with the account. Only returned by Assets endpoints. Transaction history returned by endpoints such as `/transactions/get` or `/investments/transactions/get` will be returned in the top-level `transactions` field instead.
            owners ([Owner]): Data returned by the financial institution about the account owner or owners. Only returned by Identity or Assets endpoints. Multiple owners on a single account will be represented in the same `owner` object, not in multiple owner objects within the array. In API versions 2018-05-22 and earlier, the `owners` object is not returned, and instead identity information is returned in the top level `identity` object. For more details, see [Plaid API versioning](https://plaid.com/docs/api/versioning/#version-2019-05-29)
            historical_balances ([HistoricalBalance]): Calculated data about the historical balances on the account. Only returned by Assets endpoints.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            verification_status (str): The current verification status of an Auth Item initiated through Automated or Manual micro-deposits.  Returned for Auth Items only.  `pending_automatic_verification`: The Item is pending automatic verification  `pending_manual_verification`: The Item is pending manual micro-deposit verification. Items remain in this state until the user successfully verifies the two amounts.  `automatically_verified`: The Item has successfully been automatically verified   `manually_verified`: The Item has successfully been manually verified  `verification_expired`: Plaid was unable to automatically verify the deposit within 7 calendar days and will no longer attempt to validate the Item. Users may retry by submitting their information again through Link.  `verification_failed`: The Item failed manual micro-deposit verification because the user exhausted all 3 verification attempts. Users may retry by submitting their information again through Link.   . [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        constant_args = {
            '_check_type': _check_type,
            '_path_to_item': _path_to_item,
            '_spec_property_naming': _spec_property_naming,
            '_configuration': _configuration,
            '_visited_composed_classes': self._visited_composed_classes,
        }
        required_args = {
            'account_id': account_id,
            'balances': balances,
            'mask': mask,
            'name': name,
            'official_name': official_name,
            'type': type,
            'subtype': subtype,
            'days_available': days_available,
            'transactions': transactions,
            'owners': owners,
            'historical_balances': historical_balances,
        }
        model_args = {}
        model_args.update(required_args)
        model_args.update(kwargs)
        composed_info = validate_get_composed_info(
            constant_args, model_args, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        unused_args = composed_info[3]

        for var_name, var_value in required_args.items():
            setattr(self, var_name, var_value)
        for var_name, var_value in kwargs.items():
            if var_name in unused_args and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        not self._additional_properties_model_instances:
                # discard variable.
                continue
            setattr(self, var_name, var_value)

    @cached_property
    def _composed_schemas():
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error beause the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        lazy_import()
        return {
          'anyOf': [
          ],
          'allOf': [
              AccountAssetsAllOf,
              AccountBase,
          ],
          'oneOf': [
          ],
        }



from setuptools import setup, find_packages

VERSION = '8.7.0'
url = 'https://github.com/plaid/plaid-python'

REQUIRES = [
  "urllib3 >= 1.25.3",
  "python-dateutil",
  "nulltype",
]

setup(
    name='plaid-python',
    version=VERSION,
    description='Python client library for the Plaid API and Link',
    long_description='',
    keywords='api, client, plaid',
    author='Plaid Inc.',
    author_email='developers@plaid.com',
    url=url,
    download_url='{}/tarball/v{}'.format(url, VERSION),
    license='MIT',
    packages=find_packages(exclude='tests'),
    package_data={'README': ['README.md']},
    python_requires=">=3.6",
    install_requires=REQUIRES,
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Environment :: Web Environment",
    ]
)



from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.item_remove_request import ItemRemoveRequest
from plaid.model.investments_holdings_get_request import InvestmentsHoldingsGetRequest

from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION
)

access_token = None


def setup_module(module):
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('investments')]
    )
    pt_response = client.sandbox_public_token_create(pt_request)

    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )

    exchange_response = client.item_public_token_exchange(exchange_request)

    global access_token
    access_token = exchange_response['access_token']


def teardown_module(module):
    client = create_client()
    ir_request = ItemRemoveRequest(
        access_token=access_token
    )
    client.item_remove(ir_request)


def test_get():
    client = create_client()
    request = InvestmentsHoldingsGetRequest(
        access_token=access_token
    )
    response = client.investments_holdings_get(request)
    assert response['item'] is not None
    assert response['accounts'] is not None
    assert response['securities'] is not None
    assert response['holdings'] is not None


import time
import json
import plaid
import datetime as dt
from plaid.model.products import Products
from plaid.model.sandbox_public_token_create_request import SandboxPublicTokenCreateRequest
from plaid.model.sandbox_public_token_create_request_options import SandboxPublicTokenCreateRequestOptions
from plaid.model.sandbox_public_token_create_request_options_transactions import SandboxPublicTokenCreateRequestOptionsTransactions
from plaid.model.item_public_token_exchange_request import ItemPublicTokenExchangeRequest
from plaid.model.transactions_get_request import TransactionsGetRequest
from plaid.model.transactions_get_request_options import TransactionsGetRequestOptions
from plaid.model.item_remove_request import ItemRemoveRequest
from plaid.model.transactions_refresh_request import TransactionsRefreshRequest
from tests.integration.util import (
    create_client,
    SANDBOX_INSTITUTION,
)

access_token = None

# NOTE: Data is only generated over the past 2 years.  Ensure that the date
# range used for transactions/get is within 2 years old
END_DATE = dt.date.today()
START_DATE = (END_DATE - dt.timedelta(days=(365*2)))


def setup_module(module):
    client = create_client()
    pt_request = SandboxPublicTokenCreateRequest(
        institution_id=SANDBOX_INSTITUTION,
        initial_products=[Products('transactions')],
        options=SandboxPublicTokenCreateRequestOptions(
            transactions=SandboxPublicTokenCreateRequestOptionsTransactions(
                start_date=START_DATE,
                end_date=END_DATE,
            )
        )
    )
    pt_response = client.sandbox_public_token_create(pt_request)
    exchange_request = ItemPublicTokenExchangeRequest(
        public_token=pt_response['public_token']
    )
    exchange_response = client.item_public_token_exchange(exchange_request)
    global access_token
    access_token = exchange_response['access_token']


def get_transactions_with_retries(client,
                                  access_token,
                                  account_ids=None,
                                  count=None,
                                  offset=None,
                                  num_retries=20):
    response = None
    for i in range(num_retries):
        try:
            options = TransactionsGetRequestOptions()

            if count is not None:
                options.count = count
            if offset is not None:
                options.offset = offset
            if account_ids is not None:
                options.account_ids = account_ids

            request = TransactionsGetRequest(
                access_token=access_token,
                start_date=START_DATE,
                end_date=END_DATE,
                options=options
            )
            response = client.transactions_get(request)

        except plaid.ApiException as e:
            response = json.loads(e.body)
            if response['error_code'] == 'PRODUCT_NOT_READY':
                time.sleep(5)
                continue
            else:
                raise e
        break
    return response


def refresh_transactions(client,
                         access_token):
    request = TransactionsRefreshRequest(
        access_token=access_token
    )
    return client.transactions_refresh(request)


def teardown_module(module):
    client = create_client()
    ir_request = ItemRemoveRequest(
        access_token=access_token
    )
    client.item_remove(ir_request)


def test_get():
    client = create_client()

    response = get_transactions_with_retries(client,
                                             access_token,
                                             num_retries=10)
    assert response['accounts'] is not None
    assert response['transactions'] is not None

    # get transactions for selected accounts
    account_id = response['accounts'][0]['account_id']
    response = get_transactions_with_retries(client,
                                             access_token,
                                             account_ids=[account_id],
                                             num_retries=10)
    assert response['transactions'] is not None


def test_get_with_options():
    client = create_client()
    response = get_transactions_with_retries(client,
                                             access_token,
                                             count=2,
                                             offset=1)
    assert len(response['transactions']) == 2


def test_refresh():
    client = create_client()

    response = refresh_transactions(client,
                                    access_token)
    # response should be empty (aside from request_id)
    assert response is not None
